import json
from enum import Enum


class RecordController:

    def __init__(self, client):
        self.client = client

    def write_record(self, stream_id, json_payload, record_id=None):
        record = Record()
        record.recordId = record_id
        record.dataPayload = json.dumps(json_payload)
        return self.client.call_api("POST",
                             f"/stream/{stream_id}/record/",
                             body=record.to_json()
                             )

    def read_record(self, stream_id, record_id, mark_as_read):
        read_record_request = ReadRecordRequest()
        read_record_request.recordId = record_id
        read_record_request.readRecordRequestType = ReadRecordRequestType.SINGLE
        read_record_request.markAsRead = mark_as_read
        read_record_request.recordCount = 1
        return self._read_record_request(stream_id, read_record_request)

    def read_record_hist(self, stream_id, record_id, count, mark_as_read):
        read_record_request = ReadRecordRequest()
        read_record_request.recordId = record_id
        read_record_request.readRecordRequestType = ReadRecordRequestType.SINGLE
        read_record_request.markAsRead = mark_as_read
        read_record_request.recordCount = count
        return self._read_record_request(stream_id, read_record_request)

    def read_unread_records(self, stream_id, mark_as_read, record_count):
        read_record_request = ReadRecordRequest()
        read_record_request.readRecordRequestType = ReadRecordRequestType.UNREAD
        read_record_request.markAsRead = mark_as_read
        read_record_request.recordCount = record_count
        return self._read_record_request(stream_id, read_record_request)

    def read_last_n_records(self, stream_id, mark_as_read, record_count):
        read_record_request = ReadRecordRequest()
        read_record_request.readRecordRequestType = ReadRecordRequestType.LAST_N
        read_record_request.markAsRead = mark_as_read
        read_record_request.recordCount = record_count
        return self._read_record_request(stream_id, read_record_request)

    def read_records_since_timestamp(self, stream_id, timestamp, mark_as_read, record_count):
        read_record_request = ReadRecordRequest()
        read_record_request.readRecordRequestType = ReadRecordRequestType.READ_SINCE_TIMESTAMP
        read_record_request.recordTimeStampISO = timestamp
        read_record_request.markAsRead = mark_as_read
        read_record_request.recordCount = record_count
        return self._read_record_request(stream_id, read_record_request)

    def _read_record_request(self, stream_id, read_record_request):
        read_record_request.readRecordRequestType = read_record_request.readRecordRequestType.value
        return self.client.call_api("POST",
                             f"/stream/{stream_id}/record-request",
                             body=read_record_request.to_json()
                             )


class ReadRecordRequest:

     def __init__(self):
         self.readRecordRequestType = None
         self.recordCount = None
         self.recordId = None
         self.markAsRead = None
         self.recordTimeStampISO = None
     def to_json(self):
         return self.__dict__

class ReadRecordRequestType(Enum):
    SAMPLE = 'SAMPLE'
    UNREAD = 'UNREAD'
    READ_SINCE_RECORD_ID = 'READ_SINCE_RECORD_ID'
    READ_SINCE_TIMESTAMP = 'READ_SINCE_TIMESTAMP'
    SINGLE = 'SINGLE'
    LAST_N = 'LAST_N'


class Record:

     def __init__(self):
         self.recordId = None
         self.streamId = None
         self.recordTimeStampISO = None
         self.readStatus = None
         self.dataPayload = None
     def to_json(self):
         return self.__dict__


import os
from unittest import TestCase

from streamduo.client import Client


class TestActor(TestCase):
    def test_create_machine_client(self):
        actor_controller = Client(os.getenv('AUTH_CLIENT_ID'), os.getenv('AUTH_CLIENT_SECRET')).get_actor_controller()
        create_actor_response = actor_controller.create_machine_client("client display name test",
                                                                       "client description test")
        new_client_id = create_actor_response.json()['clientId']
        assert new_client_id is not None

        ## cleanup
        delete_response = actor_controller.delete_machine_client(new_client_id)
        assert delete_response.status_code == 204

        ## check get
        get_response = actor_controller.get_machine_client(new_client_id)
        assert get_response.status_code == 404



class Actor:
    def __init__(self, client):
        self.client = client

    def get_machine_client(self, machine_client_id):
        return self.client.call_api('GET', f"/client/{machine_client_id}")

    def create_machine_client(self, client_display_name, client_description):
        body = {'clientDisplayName':client_display_name,
                'clientDescription': client_description}
        return self.client.call_api('POST',
                                    "/client",
                                    body=body)

    def delete_machine_client(self, client_id):
        return self.client.call_api('DELETE',
                                    f"/client/{client_id}")



import os
from unittest import TestCase
from streamduo.client import Client

class TestHealth(TestCase):
    def test_check_health(self):
        health_controller = Client(os.getenv('AUTH_CLIENT_ID'), os.getenv('AUTH_CLIENT_SECRET')).get_health_controller()
        print(health_controller.check_health())
        assert health_controller.check_health().text == 'OK'




from unittest import TestCase
import os
from streamduo.client import Client


class TestStream(TestCase):


    def test_create_stream(self):
        display_name = 'test_stream'
        stream_controller = Client(os.getenv('AUTH_CLIENT_ID'), os.getenv('AUTH_CLIENT_SECRET')).get_stream_controller()
        stream_request_result = stream_controller.create_stream(display_name)
        assert stream_request_result.json()['displayName'] == display_name
        ## Cleanup
        stream_controller.delete_stream(stream_request_result.json()['streamId'])

    def test_get_stream(self):
        display_name = 'test_stream'
        stream_controller = Client(os.getenv('AUTH_CLIENT_ID'), os.getenv('AUTH_CLIENT_SECRET')).get_stream_controller()
        new_stream_id = stream_controller.create_stream(display_name).json()['streamId']
        stream_request_result = stream_controller.get_stream(new_stream_id)
        assert stream_request_result.json()['streamId'] == new_stream_id
        ## Cleanup
        stream_controller.delete_stream(new_stream_id)

    def test_delete_stream(self):
        display_name = 'test_stream'
        stream_controller = Client(os.getenv('AUTH_CLIENT_ID'), os.getenv('AUTH_CLIENT_SECRET')).get_stream_controller()
        new_stream_id = stream_controller.create_stream(display_name).json()['streamId']
        stream_request_result = stream_controller.get_stream(new_stream_id)
        ## Assert was created
        assert stream_request_result.json()['streamId'] == new_stream_id
        stream_delete_result = stream_controller.delete_stream(new_stream_id)
        assert stream_delete_result.status_code == 204
        ## Assert 404
        stream_request_result_2 = stream_controller.get_stream(new_stream_id)
        assert stream_request_result_2.status_code == 404

    def test_manage_machine_clients(self):
        display_name = 'test_stream'
        stream_controller = Client(os.getenv('AUTH_CLIENT_ID'), os.getenv('AUTH_CLIENT_SECRET')).get_stream_controller()
        new_stream_id = stream_controller.create_stream(display_name).json()['streamId']
        client_display_name = 'test_client'
        add_client_response = stream_controller.add_new_machine_client_to_stream(new_stream_id, client_display_name)
        ##get new client ID
        new_client_id = None
        for i in add_client_response.json()['streamActorPermissionRecords']:
            if i['actorDisplayName'] == client_display_name:
                new_client_id = i['actorId']
                break
        assert new_client_id is not None

        ## Remove Client ID from stream permissions
        remove_client_response = stream_controller.remove_machine_client_from_stream(new_stream_id, new_client_id)
        check_client_id = None
        for i in remove_client_response.json()['streamActorPermissionRecords']:
            if i['actorDisplayName'] == client_display_name:
                check_client_id = i['actorId']
                break
        assert check_client_id is None
        assert len(remove_client_response.json()['streamActorPermissionRecords']) == 1

        ## Cleanup
        actor_controller = Client(os.getenv('AUTH_CLIENT_ID'), os.getenv('AUTH_CLIENT_SECRET')).get_actor_controller()
        actor_controller.delete_machine_client(new_client_id)
        stream_controller.delete_stream(new_stream_id)
        assert actor_controller.get_machine_client(new_client_id).status_code == 404

    def test_cannot_delete_owner(self):
        display_name = 'test_stream'
        stream_controller = Client(os.getenv('AUTH_CLIENT_ID'), os.getenv('AUTH_CLIENT_SECRET')).get_stream_controller()
        new_stream_id = stream_controller.create_stream(display_name).json()['streamId']

        user_controller = Client(os.getenv('AUTH_CLIENT_ID'), os.getenv('AUTH_CLIENT_SECRET')).get_user_controller()
        user_id = user_controller.get_user().json()['userId']
        remove_response = stream_controller.remove_user_from_stream(stream_id=new_stream_id, client_id=user_id)
        assert remove_response.status_code == 403

        ## Cleanup
        stream_controller.delete_stream(new_stream_id)

    def test_add_user(self):
        display_name = 'test_stream'
        user_email = "dev@frontend.com"
        stream_controller = Client(os.getenv('AUTH_CLIENT_ID'), os.getenv('AUTH_CLIENT_SECRET')).get_stream_controller()
        new_stream_id = stream_controller.create_stream(display_name).json()['streamId']
        #Add a user to a stream
        add_client_response = stream_controller.add_user_to_stream(stream_id=new_stream_id, user_email=user_email)
        stream_response = stream_controller.get_stream(stream_id=new_stream_id)
        user_id = None
        for i in stream_response.json()['streamActorPermissionRecords']:
            if i['actorDisplayName'] == user_email:
                user_id = i['actorId']
                break
        assert user_id is not None

        # Remove User
        remove_response = stream_controller.remove_user_from_stream(stream_id=new_stream_id, client_id=user_id)
        stream_response2 = stream_controller.get_stream(stream_id=new_stream_id)

        user_id = None
        for i in stream_response2.json()['streamActorPermissionRecords']:
            if i['actorDisplayName'] == user_email:
                user_id = i['actorId']
                break
        assert user_id is None

        ## Cleanup
        stream_controller.delete_stream(new_stream_id)




class Stream:
    def __init__(self, client):
        self.client = client

    def create_stream(self, stream_display_name):
        request_body = {'displayName': stream_display_name}
        return self.client.call_api('POST',
                                    "/stream",
                                    body=request_body)

    def get_stream(self, stream_id):
        return self.client.call_api('GET',
                                    f"/stream/{stream_id}")

    def delete_stream(self, stream_id):
        return self.client.call_api('DELETE',
                                    f"/stream/{stream_id}")

    def add_new_machine_client_to_stream(self, stream_id, client_display_name, is_producer=False, is_consumer=False ):
        body = {'actorDisplayName': client_display_name,
                'isProducer': is_producer,
                'isConsumer': is_consumer,
                'actorType': 'CLIENT'
                }
        return self.client.call_api('POST',
                                    f"/stream/{stream_id}/permissions",
                                    body=body)

    def add_user_to_stream(self, stream_id, user_email, is_producer=False, is_consumer=False):
        body = {'actorDisplayName': user_email,
                'isProducer': is_producer,
                'isConsumer': is_consumer,
                'actorType': 'USER'
                }
        return self.client.call_api('POST',
                                    f"/stream/{stream_id}/permissions",
                                    body=body)

    def remove_machine_client_from_stream(self, stream_id, client_id ):
        body = {'actorId': client_id,
                'actorType': 'CLIENT'
                }
        return self.client.call_api('DELETE',
                                    f"/stream/{stream_id}/permissions",
                                    body=body)

    def remove_user_from_stream(self, stream_id, client_id ):
        body = {'actorId': client_id,
                'actorType': 'USER'
                }
        return self.client.call_api('DELETE',
                                    f"/stream/{stream_id}/permissions",
                                    body=body)


import requests
from streamduo.api.health import Health
from streamduo.api.stream import Stream
from streamduo.api.actor import Actor
from streamduo.api.record import RecordController
from streamduo.api.user import User

class Client:
    def __init__(self, client_id, client_secret):
        """Constructor"""
        self.auth_endpoint = "https://login.streamduo.com/oauth/token"
        self.api_endpoint = "https://api.streamduo.com"

        self.client_id = client_id
        self.client_secret = client_secret
        self.token = None
        self.auth_req_header = {'content-type': 'application/x-www-form-urlencoded'}
        self.token_req_payload = {'grant_type': 'client_credentials',
                             'client_id': self.client_id,
                             'client_secret': self.client_secret,
                             'audience': 'https://api.streamduo.com'}
        self.set_oauth_token()

    def set_oauth_token(self):
        try:
            token_response = requests.post(self.auth_endpoint,
                                           data=self.token_req_payload,
                                           headers=self.auth_req_header)
            self.token = token_response.json()['access_token']
        except:
            self.token = None


    def call_api(self, verb, path, body=None):
        header = {'authorization': f"Bearer {self.token}",
                'content-type': 'application/json'}
        if verb == 'GET':
            return requests.get(f"{self.api_endpoint}{path}",
                                headers=header)
        if verb == 'POST':
            return requests.post(f"{self.api_endpoint}{path}",
                                headers=header,
                                json=body)
        if verb == 'DELETE':
            return requests.delete(f"{self.api_endpoint}{path}",
                                headers=header,
                                json=body)

    def get_health_controller(self):
        return Health(self)

    def get_stream_controller(self):
        return Stream(self)

    def get_actor_controller(self):
        return Actor(self)

    def get_record_controller(self):
        return RecordController(self)

    def get_user_controller(self):
        return User(self)
